@@ -421,7 +421,7 @@ var PlacesUIUtils = {
    * @return true if any transaction has been performed.
    *
    * Notes:
-   *  - the location, description and "load in sidebar" fields are
+   *  - the location, description and "loadInSidebar" fields are
    *    visible only if there is no initial URI (aURI is null).
    *  - When aDefaultInsertionPoint is not set, the dialog defaults to the
    *    bookmarks root folder.
@@ -489,7 +489,7 @@ var PlacesUIUtils = {
     var info = {
       action: "add",
       type: "bookmark",
-      hiddenRows: ["location", "description", "load in sidebar"]
+      hiddenRows: ["location", "description", "loadInSidebar"]
     };
     if (aURI)
       info.uri = aURI;
@@ -636,33 +636,19 @@ var PlacesUIUtils = {
   },
 
   /**
-   * Opens the bookmark properties panel for a given bookmark identifier.
+   * Opens the properties dialog for a given item identifier.
    *
-   * @param aId
-   *        bookmark identifier for which the properties are to be shown
+   * @param aItemId
+   *        item identifier for which the properties are to be shown
+   * @param aType
+   *        item type, either "bookmark" or "folder"
    * @return true if any transaction has been performed.
    */
-  showBookmarkProperties: function PU_showBookmarkProperties(aId) {
+  showItemProperties: function PU_showItemProperties(aItemId, aType) {
     var info = {
       action: "edit",
-      type: "bookmark",
-      bookmarkId: aId
-    };
-    return this._showBookmarkDialog(info);
-  },
-
-  /**
-   * Opens the folder properties panel for a given folder ID.
-   *
-   * @param aId
-   *        an integer representing the ID of the folder to edit
-   * @return true if any transaction has been performed.
-   */
-  showFolderProperties: function PU_showFolderProperties(aId) {
-    var info = {
-      action: "edit",
-      type: "folder",
-      folderId: aId
+      type: aType,
+      itemId: aItemId
     };
     return this._showBookmarkDialog(info);
   },
@@ -724,9 +710,9 @@ var PlacesUIUtils = {
 
     var features;
     if (aMinimalUI)
-//@line 734 "/export/home/mozilla/uild/firefox-nightly/src/mozilla/browser/components/places/content/utils.js"
+//@line 720 "/export/home/mozilla/uild/firefox-nightly/src/mozilla/browser/components/places/content/utils.js"
       features = "centerscreen,chrome,dialog,resizable,dependent";
-//@line 736 "/export/home/mozilla/uild/firefox-nightly/src/mozilla/browser/components/places/content/utils.js"
+//@line 722 "/export/home/mozilla/uild/firefox-nightly/src/mozilla/browser/components/places/content/utils.js"
     else
       features = "centerscreen,chrome,modal,resizable=no";
     window.openDialog(dialogURL, "",  features, aInfo);
@@ -1019,10 +1005,10 @@ var PlacesUIUtils = {
         var popup = document.createElement("menupopup");
         popup.setAttribute("placespopup", "true");
         popup._resultNode = asContainer(aNode);
-//@line 1029 "/export/home/mozilla/uild/firefox-nightly/src/mozilla/browser/components/places/content/utils.js"
+//@line 1015 "/export/home/mozilla/uild/firefox-nightly/src/mozilla/browser/components/places/content/utils.js"
         // no context menu on mac
         popup.setAttribute("context", "placesContext");
-//@line 1032 "/export/home/mozilla/uild/firefox-nightly/src/mozilla/browser/components/places/content/utils.js"
+//@line 1018 "/export/home/mozilla/uild/firefox-nightly/src/mozilla/browser/components/places/content/utils.js"
         element.appendChild(popup);
         if (aContainersMap)
           aContainersMap.push({ resultNode: aNode, domNode: popup });
